
PhysRenderer = Class("PhysRenderer")

function HSL(h, s, l, a)
    if s<=0 then return l,l,l,a end
    h, s, l = h/256*6, s/255, l/255
    local c = (1-math.abs(2*l-1))*s
    local x = (1-math.abs(h%2-1))*c
    local m,r,g,b = (l-.5*c), 0,0,0
    if h < 1     then r,g,b = c,x,0
    elseif h < 2 then r,g,b = x,c,0
    elseif h < 3 then r,g,b = 0,c,x
    elseif h < 4 then r,g,b = 0,x,c
    elseif h < 5 then r,g,b = x,0,c
    else              r,g,b = c,0,x
    end 

    return (r+m)*255/255,(g+m)*255/255,(b+m)*255/255,a/255
end


function PhysRenderer:initialize(world)
	self.world = world
	self.scale = 1

	self.show = {
		joints = true,
		bodies = true
	}
end

function PhysRenderer:draw()
	if self.world == nil then print("no world for physrenderer") return end
	love.graphics.push()

	love.graphics.setLineStyle("smooth")

	if self.show.bodies then
		self:DrawBodies()
	end
	
	if self.show.joints then
		local joints = self.world:getJoints()
		for i, joint in ipairs(joints) do
			local type = joint:getType()
			local x1, y1, x2, y2 = joint:getAnchors()
			love.graphics.setColor(255/255, 255/255, 255/255, 100/255)
			self:drawX(x1 * self.scale, y1 * self.scale, 6)
			self:drawX(x2 * self.scale, y2 * self.scale, 6)
		end
	end

	love.graphics.pop()
end

function PhysRenderer:DrawBodies()
	local bodies = self.world:getBodies()
	for i, body in ipairs(bodies) do

		local wx, wy = body:getPosition()
		local shapeColour = {HSL(i * 20 % 255, 255, 100, 255)}
		local fadedColour = {HSL(i * 20 % 255, 100, 100, 150)}
		local fillColour = {HSL(i * 20 % 255, 100, 100, 50)}
		love.graphics.setColor(shapeColour)

		-- Translate to local space
		love.graphics.push()
		love.graphics.translate(wx * self.scale, wy * self.scale)

		local fixtures = body:getFixtures()		
		love.graphics.rotate(body:getAngle())
		for _, fixture in ipairs(fixtures) do			
			if true or not fixture:getUserData().bHidden then
				local shape = fixture:getShape()
				local shapeType = shape:type()

				if shapeType == "ChainShape" then
					-- WARNING: In 0.8 this will crash:
					-- ChainShape:getPoints()				
					local points = {shape:getPoints()}
					for i=1, #points, 2 do
						local x = points[i]
						local y = points[i + 1]
						-- Scale points
						points[i] = x * self.scale
						points[i + 1] = y * self.scale
						love.graphics.circle("fill", x * self.scale, y * self.scale, 2)
					end

					love.graphics.setColor(fadedColour)
					love.graphics.line(unpack(points))

				elseif shapeType == "CircleShape" then
					local radius = shape:getRadius()
					if fixture:isSensor() then
						love.graphics.setColor(fadedColour)
						love.graphics.circle("fill", 0, 0, radius)
					end			
					love.graphics.setColor(fadedColour)
					love.graphics.circle("line", 0, 0, radius)
					love.graphics.setColor(shapeColour)
				elseif shapeType == "PolygonShape" then
					local points = {shape:getPoints()}

					if fixture:isSensor() then
						love.graphics.setColor(fadedColour)
						love.graphics.polygon("fill", unpack(points))					
	
						local ud = fixture:getUserData()
						if ud and ud.isSensor and ud.isSensisngWorld then
							love.graphics.setColor(255, 255, 255)
						end
					end
					for i=1, #points, 2 do
						local x = points[i]
						local y = points[i + 1]
						-- Scale points
						points[i] = x * self.scale
						points[i + 1] = y * self.scale
						love.graphics.circle("fill", x * self.scale, y * self.scale, 2)
					end
					-- Draw outline
					love.graphics.setColor(fadedColour)
					love.graphics.line(unpack(points))
					--- Draw fill 
					love.graphics.setColor(fillColour)
					love.graphics.polygon("fill", unpack(points))
					-- Close shape
					love.graphics.setColor(fadedColour)
					love.graphics.line(points[#points-1], points[#points], points[1], points[2])			
					
				end			
			end
		end

		love.graphics.pop()
	end
end

function PhysRenderer:drawgame()
	if self.world == nil then print("no world for physrenderer") return end
	love.graphics.push()

	love.graphics.setLineStyle("smooth")

	local bodies = self.world:getBodyList()
	for i, body in ipairs(bodies) do

		local wx, wy = body:getPosition()
		local shapeColour = {HSL(i * 20 % 255/255, 255, 100, 255)}
		local fadedColour = {HSL(i * 20 % 255/255, 100, 100, 150)}
		local fillColour = {HSL(i * 20 % 255/255, 100, 100, 50)}
		love.graphics.setColor(shapeColour)

		-- Translate to local space
		love.graphics.push()
		love.graphics.translate(wx * self.scale, wy * self.scale)

		local fixtures = body:getFixtures()		
		love.graphics.rotate(body:getAngle())
		for _, fixture in ipairs(fixtures) do			
			if true or not fixture:getUserData().bHiddenGame then
				local shape = fixture:getShape()
				local shapeType = shape:type()

				if shapeType == "ChainShape" then
					-- WARNING: In 0.8 this will crash:
					-- ChainShape:getPoints()				
					local points = {shape:getPoints()}
					for i=1, #points, 2 do
						local x = points[i]
						local y = points[i + 1]
						-- Scale points
						points[i] = x * self.scale
						points[i + 1] = y * self.scale
						love.graphics.circle("fill", x * self.scale, y * self.scale, 2)
					end

					love.graphics.setColor(fadedColour)
					love.graphics.line(unpack(points))

				elseif shapeType == "CircleShape" then
					local radius = shape:getRadius()
					if fixture:isSensor() then
						love.graphics.setColor(fadedColour)
						love.graphics.circle("fill", 0, 0, radius)
					end			
					love.graphics.setColor(fadedColour)
					love.graphics.circle("line", 0, 0, radius)
					love.graphics.setColor(shapeColour)
				elseif shapeType == "PolygonShape" then
					local points = {shape:getPoints()}

					if fixture:isSensor() then						
						love.graphics.setColor(fadedColour)
						love.graphics.polygon("fill", unpack(points))					
	
						local ud = fixture:getUserData()
						if ud and ud.isSensor and ud.isSensisngWorld then
							love.graphics.setColor(255, 255, 255)
						end
					end
					for i=1, #points, 2 do
						local x = points[i]
						local y = points[i + 1]
						-- Scale points
						points[i] = x * self.scale
						points[i + 1] = y * self.scale
						love.graphics.circle("fill", x * self.scale, y * self.scale, 2)
					end
					-- Draw outline
					love.graphics.setColor(fadedColour)
					love.graphics.line(unpack(points))
					--- Draw fill 
					love.graphics.setColor(fillColour)
					love.graphics.polygon("fill", unpack(points))
					-- Close shape
					love.graphics.setColor(fadedColour)
					love.graphics.line(points[#points-1], points[#points], points[1], points[2])			
					
				end
			end
		end

		love.graphics.pop()
	end

	local joints = self.world:getJoints()
	for i, joint in ipairs(joints) do
		local type = joint:getType()
		local x1, y1, x2, y2 = joint:getAnchors()
		love.graphics.setColor(255, 255, 255, 100)
		self:drawX(x1 * self.scale, y1 * self.scale, 6)
		self:drawX(x2 * self.scale, y2 * self.scale, 6)
	end

	love.graphics.pop()
end

function PhysRenderer:drawX(x, y, radius)
	love.graphics.line(x - radius, y - radius, x + radius, y + radius) -- \
	love.graphics.line(x + radius, y - radius, x - radius, y + radius) -- /
end

